package com.usky.issue.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.StringUtils;
import com.usky.issue.domain.SpHj2017;
import com.usky.issue.domain.SpOwner;
import com.usky.issue.domain.SpRtu2017;
import com.usky.issue.domain.SpSj2017;
import com.usky.issue.mapper.SpSj2017Mapper;
import com.usky.issue.service.SpOwnerService;
import com.usky.issue.service.SpSj2017Service;
import com.usky.common.mybatis.core.AbstractCrudService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * asf 服务实现类
 * </p>
 *
 * @author fu
 * @since 2023-12-13
 */
@Service
@DS("jdxf")
public class SpSj2017ServiceImpl extends AbstractCrudService<SpSj2017Mapper, SpSj2017> implements SpSj2017Service {
    @Autowired
    private SpOwnerService spOwnerService;

    /**
     * 设备状态统计
     * @param requestBody
     * @return
     */
    @Override
    public List<Map<String, Object>> statusCount(String requestBody) {
        JSONObject requestVO = JSONObject.parseObject(requestBody);
        String companyCode = requestVO.get("companyCode").toString();
        Integer currentPage = (Integer) requestVO.get("current");
        Integer pageSize = (Integer) requestVO.get("size");
        int startIndex = (currentPage - 1) * pageSize;
        QueryWrapper<SpSj2017> queryWrapper = Wrappers.query();
        queryWrapper.select("DATE_FORMAT(time, '%Y-%m-%d %H:%i:%s') AS alarm_time","data1 AS alarm_type","data4 AS alarm_point " ,"device_code")
                .inSql("device_code", "SELECT owner_code FROM sp_owner WHERE company = '" + companyCode + "' AND dwtype IN (2,5)");
        queryWrapper.last("LIMIT " + pageSize + " OFFSET " + startIndex)
                    .orderByDesc("id");
        List<Map<String, Object>> deviceStatusList = this.listMaps(queryWrapper);
        QueryWrapper<SpOwner> queryWrapper1 = Wrappers.query();
        queryWrapper1.select("owner_code","unitinfo")
                .eq("company",companyCode)
                .in("dwtype",2,5);
        List<Map<String, Object>> deviceInfoList = spOwnerService.listMaps(queryWrapper1);
        if(CollectionUtils.isNotEmpty(deviceStatusList)){
            for (int i=0;i<deviceStatusList.size();i++){
                if(CollectionUtils.isNotEmpty(deviceInfoList)){
                    for (int j=0;j<deviceInfoList.size();j++){
                        if(deviceStatusList.get(i).get("device_code").equals(deviceInfoList.get(j).get("owner_code"))){
                            deviceStatusList.get(i).put("device_address",deviceInfoList.get(j).get("unitinfo"));
                            break;
                        }
                    }
                }
            }
        }
        return deviceStatusList;
    }

    /**
     * 设备查询
     * @param requestBody
     * @return
     */
    public List<Map<String, Object>> typeList(String requestBody) {
        JSONObject requestVO = JSONObject.parseObject(requestBody);
        String companyCode = requestVO.get("companyCode").toString();
        QueryWrapper<SpSj2017> queryWrapper1 = Wrappers.query();
        queryWrapper1.select("COUNT(CASE WHEN clzt = 0 THEN 1 ELSE NULL END) AS untreated", "COUNT(CASE WHEN clzt = 1 THEN 1 ELSE NULL END) AS processed")
                .inSql("device_code", "SELECT owner_code FROM sp_owner WHERE company = '" + companyCode + "' AND dwtype IN (2,5)");
        List<Map<String, Object>> deviceStatusList1 = this.listMaps(queryWrapper1);
        return deviceStatusList1;
    }

    /**
     * 设备数量统计
     * @param requestBody
     * @return
     */
    public Integer countDevice(String requestBody) {
        JSONObject requestVO = JSONObject.parseObject(requestBody);
        String companyCode = requestVO.get("companyCode").toString();
        QueryWrapper<SpSj2017> queryWrapper1 = Wrappers.query();
        queryWrapper1.select("DISTINCT device_code")
                .inSql("device_code", "SELECT owner_code FROM sp_owner WHERE company = '" + companyCode + "'");
        List<SpSj2017> SpSj2017 = this.list(queryWrapper1);
        Integer deviceCount = SpSj2017.size();
        return deviceCount;
    }

    /**
     * 告警信息更新
     * @return
     */
    @Override
    public List<SpSj2017> getOwners() {
        List<String> codes = spOwnerService.getOwners(spOwnerService.getDeviceCodes())
                .stream()
                .filter(spOwner -> spOwner.getDwtype()==2 || spOwner.getDwtype()==5)
                .map(SpOwner::getOwnerCode)
                .collect(Collectors.toList());
        return baseMapper.selectList(Wrappers.<SpSj2017>query()
                .select("device_code", "time", "data1","clr",
                        "IF(clsj = '0000-00-00 00:00:00', '1997-01-01 00:00:00', clsj) AS clsj",
                        "clnr", "clzt", "clwb")
                .in("device_code", codes)
        );
    }

    @Override
    public String getAlarmName(String data1) {
        switch (data1) {
            case "WP1":
                return "低压";
            case "WP2":
                return "高压";
            case "WP3":
                return "故障";
            case "LL1":
                return "低液位";
            case "LL2":
                return "高液位";
            case "LL3":
                return "故障";
            default:
                return "";
        }
    }
}
